@@ -1,4 +1,6 @@
 #!/usr/bin/perl
+#
+# Check database for inconsistent entries and remove them
 
 package HaCi::HaCi;
 
@@ -16,7 +18,7 @@ $ENV{workdir} = $workDir;
 
 use Getopt::Std;
 use Data::Dumper qw/Dumper/;
-use HaCi::Conf;
+use HaCi::Conf qw/getConfigValue/;
 use HaCi::Utils qw/getConfig getPlugins getTable initTables finalizeTables initCache/;
 
 $Getopt::Std::STANDARD_HELP_VERSION = 1;
@@ -24,6 +26,11 @@ $main::VERSION                      = 0.1;
 our $opts														= {};
 getopts('cd', $opts);
 
+my $mysqlDumpBin				= 'mysqldump';
+my $mysqlDumpArgs				= q{-u'<USER>' -p'<PASS>' -h'<HOST>' '<DBNAME>'};
+my $postgresqlDumpBin		= 'pg_dump';
+my $postgresqlDumpArgs	= q{-U'<USER>' -h'<HOST>' '<DBNAME>'};
+
 my $debug							= (exists $opts->{d}) ? $opts->{d} : 0;
 my @busy							= ('|', '/', '-', '\\', '-');
 my ($bc, $ec, $sc)		= (0, 0, 0);
@@ -113,8 +120,20 @@ sub main {
 
 	if ($opts->{c}) {
 		print "\nPlease make sure to make a database backup before cleaning it up.\n";
-		if (system("which mysqldump >/dev/null") == 0) {
-			my $md			= qx/which mysqldump/;
+		my $dbType		= &getConfigValue('db', 'dbtype');
+		my $dumpBin		= '';
+		my $dumpArgs	= '';
+		if ($dbType eq 'mysql') {
+			$dumpBin	= $mysqlDumpBin;
+			$dumpArgs	= $mysqlDumpArgs;
+		}
+		elsif ($dbType eq 'postgresql') {
+			$dumpBin	= $postgresqlDumpBin;
+			$dumpArgs	= $postgresqlDumpArgs;
+		}
+
+		if ($dumpBin && system("which $dumpBin >/dev/null") == 0) {
+			my $md			= qx/which $dumpBin/;
 			chomp($md);
 			my $dbConf	= $conf->{user}->{db};
 			my $pwd			= qx/pwd/;
@@ -125,12 +144,23 @@ sub main {
 			while ( -f $dbname ) {
 				$dbname	= $dbnameO . '.' . $cnter++;
 			}
-			my $backupT	= $md . " -u'$dbConf->{dbuser}' -p'XXX' -h'$dbConf->{dbhost}' '$dbConf->{dbname}' > $dbname";
-			print "Found 'mysqldump':\n\t$backupT\nShould I run this dump? [y|N]: ";
+			#my $backupT	= $md . " -u'$dbConf->{dbuser}' -p'XXX' -h'$dbConf->{dbhost}' '$dbConf->{dbname}' > $dbname";
+			my $backupT	= $md . ' ' . $dumpArgs . "> $dbname";
+			$backupT		=~ s/<USER>/$dbConf->{dbuser}/g;
+			$backupT		=~ s/<PASS>/XXX/g;
+			$backupT		=~ s/<HOST>/$dbConf->{dbhost}/g;
+			$backupT		=~ s/<DBNAME>/$dbConf->{dbname}/g;
+
+			print "Found '$dumpBin':\n\t$backupT\nShould I run this dump? [y|N]: ";
 			my $bb	= <STDIN>;
 			chomp($bb);
 			if (lc($bb) eq 'y') {
-				my $backupStr	= $md . " -u'$dbConf->{dbuser}' -p'$dbConf->{dbpass}' -h'$dbConf->{dbhost}' '$dbConf->{dbname}' > $dbname";
+				#my $backupStr	= $md . " -u'$dbConf->{dbuser}' -p'$dbConf->{dbpass}' -h'$dbConf->{dbhost}' '$dbConf->{dbname}' > $dbname";
+				my $backupStr	= $md . ' ' . $dumpArgs . "> $dbname";
+				$backupStr		=~ s/<USER>/$dbConf->{dbuser}/g;
+				$backupStr		=~ s/<PASS>/$dbConf->{dbpass}/g;
+				$backupStr		=~ s/<HOST>/$dbConf->{dbhost}/g;
+				$backupStr		=~ s/<DBNAME>/$dbConf->{dbname}/g;
 				print qx($backupStr);
 				if ($?) {
 					print "... Dump was NOT successful!!!\n"; 
@@ -139,6 +169,7 @@ sub main {
 				}
 			}
 		}
+
 		print "Continue cleaning up database? [y|N]: ";
 		my $con	= <STDIN>;
 		print "\n";
@@ -196,7 +227,8 @@ sub checkRootStuff {
 	}
 
 	# Get stale V4
-	my @networksNR	= $networkTable->search(['ID', 'network', 'description', 'rootID', 'ipv6ID'], {}, 0, "WHERE rootID NOT IN ($roots)");
+	my $nriCol			= $networkTable->meth2Col('rootID');
+	my @networksNR	= $networkTable->search(['ID', 'network', 'description', 'rootID', 'ipv6ID'], {}, 0, "WHERE $nriCol NOT IN ($roots)");
 	foreach (@networksNR) {
 		my $ip	= $_;
 		print "\r" . $busy[$bc++%5];
@@ -206,7 +238,8 @@ sub checkRootStuff {
 	print "\r " . ($#networksNR + 1) . " stale V4 network entrys found without matching root\n";
 
 	# Get stale V6
-	my @networkV6sNR	= $networkV6Table->search(['ID', 'rootID'], {}, 0, "WHERE rootID NOT IN ($roots)");
+	my $nriV6Col			= $networkV6Table->meth2Col('rootID');
+	my @networkV6sNR	= $networkV6Table->search(['ID', 'rootID'], {}, 0, "WHERE $nriV6Col NOT IN ($roots)");
 	foreach (@networkV6sNR) {
 		my $ip	= $_;
 		print "\r" . $busy[$bc++%5];
@@ -215,18 +248,9 @@ sub checkRootStuff {
 	}
 	print "\r " . ($#networkV6sNR + 1) . " stale V6 network entrys found without matching root\n";
 
-	# Get stale networkAC
-	my @networkACsNR	= $networkACTable->search(['ID', 'rootID'], {}, 0, "WHERE rootID NOT IN ($roots)");
-	foreach (@networkACsNR) {
-		my $ac	= $_;
-		print "\r" . $busy[$bc++%5];
-		print "\r $ac->{ID} : $ac->{rootID}: Stale network ACL entry (root doesn't exists)\n" if $debug;
-		&delEntry($networkACTable, $ac->{ID});
-	}
-	print "\r " . ($#networkACsNR + 1) . " stale network ACL entrys found without matching root\n";
-
 	# Get stale rootAC
-	my @rootACsNR	= $rootACTable->search(['ID', 'rootID'], {}, 0, "WHERE rootID NOT IN ($roots)");
+	my $rriCol		= $rootACTable->meth2Col('rootID');
+	my @rootACsNR	= $rootACTable->search(['ID', 'rootID'], {}, 0, "WHERE $rriCol NOT IN ($roots)");
 	foreach (@rootACsNR) {
 		my $ac	= $_;
 		print "\r" . $busy[$bc++%5];
