@@ -12,9 +12,11 @@ use SOAP::Lite
 		die (defined $res && ref $res ? $res->faultstring : $soap->transport->status), "\n";
 	};
 
-my $haciApiUrl	= 'http://demo.haci.larsux.de/cgi-bin/HaCiAPI.cgi?getWSDL';
-my $user		= 'admin';
-my $pass		= 'admin';
+my $haciApiUrl	= 'http://hacidev.larsux.de/cgi-bin/HaCiAPI.cgi?getWSDL';
+#my $user		= 'admin';
+#my $pass		= 'admin';
+my $user		= 'test';
+my $pass		= '1';
 my $a			= $ARGV[0] || 0;
 
 my $soap	= SOAP::Lite->service($haciApiUrl);
