@@ -86,6 +86,7 @@ function checkIfIPv6 (rootID, source) {
 				netmaskBlock.style.display	= 'none';
 			} else {
 				document.getElementById('netmask').disabled	= true;
+				document.getElementById('netmask').value	= '';
 			}
 			document.getElementsByName('netaddress')[0].size			= 43;
 			document.getElementsByName('netaddress')[0].maxLength	= 43;
@@ -201,6 +202,16 @@ function setNetmask (cidr, target_netmask, target_netaddress, target_cidr) {
 
 function setnetmask_cidr (network, target_netmask, target_cidr, target_netaddress) {
 	var splits	= network.split('/');
+
+	if (splits[1] == null || splits[1] == '') {
+		if (ipv6 == 1) {
+			splits[1] = 128
+		} else {
+			splits[1] = 32
+		}
+	}
+console.log(splits);
+
 	if (ipv6 == 1) {
 		splits[0]	= adjustIPv6(splits[0]);
 		if (splits[1]) {
@@ -434,15 +445,22 @@ function updTmplParamsFromPreview (ID, type, pos) {
 			break;
 		case 3:
 			var optionStr	= '';
-			for (i = 0; i < document.getElementsByTagName("option").length; i++) {
-				var tagTitle	= document.getElementsByTagName("option")[i].title;
-				if (tagTitle == 'tmplEntryID_' + ID) {
-					var tagValue	= document.getElementsByTagName("option")[i].value;
-					if (optionStr != '')
-						optionStr	+= ';';
-					optionStr	+= tagValue;
+			for (i = 0; i < document.getElementsByTagName("select").length; i++) {
+				var selectName	= document.getElementsByTagName("select")[i].name;
+				if (selectName == 'tmplEntryID_' + ID) {
+					var options	= document.getElementsByTagName("select")[i].options;
+					for (u = 0; u < options.length; u++) {
+						if (options[u] != undefined) {
+							var optionValue	= options[u].value;
+							if (optionStr != '')
+								optionStr	+= ';';
+							optionStr	+= optionValue;
+						}
+					}
+					break;
 				}
 			}
+
 			document.getElementById('TmplEntryParamDescr').value			= document.getElementById('tmplEntryDescrID_' + ID).value;
 			document.getElementById('TmplEntryParamEntries').disabled	= '';
 			document.getElementById('TmplEntryParamEntries').value		= optionStr;
@@ -492,6 +510,34 @@ function setACLs (id, acl) {
 	}
 }
 
+function setAllACLs (source, acl) {
+
+	var sourceChecked	= document.getElementById(source).checked;
+	var regex		= new RegExp('^accGroup_' + acl + '_');
+	var i				= 0;
+	var inputs	= document.getElementsByTagName('input');
+	for (i = 0; i < inputs.length; ++i) {
+		if (!inputs[i].name.match(regex))
+			continue;
+		var index	= inputs[i].name.replace(regex, '');
+		inputs[i].checked	= sourceChecked;
+		setACLs(index, acl);
+	}
+
+	switch (acl) {
+		case 'r':
+			if (!sourceChecked) {
+				document.getElementById('checkAll_write').checked	= false;
+			}
+			break;
+		case 'w':
+			if (sourceChecked) {
+				document.getElementById('checkAll_read').checked	= true;
+			}
+			break;
+	}
+}
+
 function chkTmplPosition (max) {
 	if (max != undefined) {
 		maxTmplPositions	= max;
@@ -723,3 +769,66 @@ function clearTextfield(id) {
 	jumpto.value				= '';
 	jumpto.style.color	= '#000000'
 }
+
+function changeACLFromParent() {
+	var aclFromParentCB	= document.getElementsByName("aclsFromParent")[0];
+	document.getElementsByName("checkAll_read")[0].disabled	= aclFromParentCB.checked;
+	document.getElementsByName("checkAll_write")[0].disabled	= aclFromParentCB.checked;
+
+	var regex		= new RegExp('^accGroup_');
+	var i				= 0;
+	var inputs	= document.getElementsByTagName('input');
+	for (i = 0; i < inputs.length; ++i) {
+		if (!inputs[i].name.match(regex))
+			continue;
+		inputs[i].disabled	= aclFromParentCB.checked;
+	}
+}
+
+function enableACLs() {
+	var regex		= new RegExp('^accGroup_');
+	var i				= 0;
+	var inputs	= document.getElementsByTagName('input');
+	for (i = 0; i < inputs.length; ++i) {
+		if (!inputs[i].name.match(regex))
+			continue;
+		inputs[i].disabled	= 0
+	}
+}
+
+function addAvailTag() {
+	var tags			= document.getElementById("tags");
+	var availTags	= document.getElementById("availTags");
+	var newTag		= availTags.selectedOptions[0].text;
+
+	var exists	= 0;
+	tags.value.split(/\s+/).forEach(function(tag) {
+		if (tag == newTag) {
+			window.alert("Tag '" + tag + "' is already set");
+			exists	= 1;
+		}
+	});
+
+	if (exists)
+		return;
+
+	if (tags.value != '')
+		tags.value += " ";
+	
+	tags.value	+= newTag;
+}
+
+function toggleEnableIntAcc() {
+	var checkBox	= document.getElementsByName("enableInternalAccount")[0];
+	var pass1	= document.getElementsByName("password1")[0];
+	var pass2	= document.getElementsByName("password2")[0];
+	if (checkBox.checked) {
+		pass1.disabled	= false;
+		pass2.disabled	= false;
+	} else {
+		pass1.disabled	= true;
+		pass1.value			= '';
+		pass2.disabled	= true;
+		pass2.value			= '';
+	}
+}
