@@ -27,7 +27,7 @@ sub _gettext {
 }
 
 sub initLocale {
-		my $locale	= shift;
+		my $locale	= shift || 'C';
 
 		setlocale(LC_ALL, $locale);
 		$conf->{var}->{GETTEXT}	= Locale::gettext->domain_raw('HaCi');
