@@ -26,13 +26,13 @@ use HaCi::Mathematics qw/
 	netv6Dec2net
 /;
 use HaCi::Utils qw/
-	prWarnl delUser getRights importCSV compare checkDB newWindow prNewWindows setStatus removeStatus
+	prWarnl delUser getRights importCSV compare checkDB newWindow prNewWindows setStatus removeStatus netv6Dec2ipv6ID
 	addRoot addNet importASNRoutes getConfig checkSpelling_Net rootID2Name rootName2ID delNet genRandBranch editRoot
 	delRoot copyNetsTo delNets search checkSpelling_IP checkSpelling_CIDR saveTmpl delTmpl saveGroup delGroup saveUser
 	lwd dec2bin groupID2Name checkRight importDNSTrans importDNSLocal getNetworkParentFromDB importConfig rootID2ipv6
-	expand splitNet combineNets updatePluginDB getTable checkTables checkNetworkACTable checkNetACL getRoots
-	mkPluginConfig initTables initCache chOwnPW updSettings updateSettingsInSession flushACLCache flushNetCache
-	searchAndGetFreeSubnets getParam
+	expand splitNet combineNets updatePluginDB getTable checkTables checkNetworkTable checkNetworkACTable checkNetACL getRoots
+	mkPluginConfig initTables initCache chOwnPW updSettings updateSettingsInSession flushACLCache flushNetCache exportSubnets
+	searchAndGetFreeSubnets getParam changeTmplName checkNetworkLock getFreeSubnets getNetID getDBNetworkBefore
 /;
 
 local our $q				= undef;
@@ -63,15 +63,16 @@ sub run {
 		use Data::Dumper qw/Dumper/; # Loading Dumper for Debugging...
 		foreach ($q->param) {
 			my $k	= $_;
+			next if $k eq 'password';
 			foreach ($q->param($k)) {
 				warn "  $k => $_ \n"; 
 			}
 		}
 	} # Dump
 
-	if ($q->param('func')) {
-		if ($q->param('func') eq 'logout') {
-			if ($session->param('authenticated')) {
+	if (&getParam(1, 0, 'func')) {
+		if (&getParam(1, '', 'func') eq 'logout') {
+			if (defined $session && $session->param('authenticated')) {
 				&debug("logging out");
 				&removeStatus();
 				$session->delete();
@@ -89,21 +90,31 @@ sub run {
 
 	my $t	= $HaCi::GUI::init::t;
 
-	if (!$conf->{var}->{relogin} && &authentication()) {
+	if (
+		!$conf->{var}->{relogin} && 
+		&authentication(
+			&getParam(1, undef, 'username', 0, 1), 
+			&getParam(1, undef, 'password'), 
+			&getParam(1, undef, 'login'), 
+			$session->param('username')
+		)
+	) {
 		&getRights();
 		&$soapPreRun($soapArgs, $q) if $soap && defined $soapPreRun;
 		&checkRights();
 		&main();
 	} else {
 		&checkTables();
+		&checkNetworkTable();
 		&checkNetworkACTable();
 		&HaCi::GUI::authentication::login();
 		$t->{V}->{page}  = 'login';
 	}
 
 	my $warnings	= '';
+	my $localQ		= $q;
 	$warnings			= &finalize();
-	return &$soapPostRun($soapArgs, $q, $t, $warnings) if $soap && defined $soapPostRun;
+	return &$soapPostRun($soapArgs, $localQ, $t, $warnings) if $soap && defined $soapPostRun;
 }
 
 sub main {
@@ -111,70 +122,73 @@ sub main {
 
 	&HaCi::GUI::init::setUserVars();
 
-	if ($q->param('fname')) {	# AJAX call
+	if (&getParam(1, 0, 'fname')) {	# AJAX call
 		return;
 	}
 	
 	my $t	= $HaCi::GUI::init::t;
 	my $q	= $HaCi::HaCi::q;
 
-	if ($q->param('submitAddRoot')) {
+	if (&getParam(1, 0, 'submitAddRoot')) {
 		if (&addRoot(
-			&getParam(1, undef, $q->param('name')),
-			&getParam(1, undef, $q->param('descr')),
-			&getParam(1, 0, $q->param('ipv6')),
-			&getParam(1, undef, $q->param('rootID')),
+			&getParam(1, undef, 'name'),
+			&getParam(1, undef, 'descr'),
+			&getParam(1, 0, 'ipv6'),
+			&getParam(1, undef, 'rootID'),
 		)) {
 			$q->delete('func'); $q->param('func', 'showAllNets');
 		} else {
 			$q->delete('func'); $q->param(
 				'func', 
-				((&getParam(1, 0, $q->param('editRoot'))) ? 'editRoot' : 'addRoot')
+				((&getParam(1, 0, 'editRoot')) ? 'editRoot' : 'addRoot')
 			);
 		}
 	}
-	elsif ($q->param('submitAddNet')) {
+	elsif (&getParam(1, 0, 'submitAddNet')) {
 		if (&addNet(
-			&getParam(1, 0, $q->param('netID')),
-			&getParam(1, undef, $q->param('rootID')),
-			&getParam(1, undef, $q->param('netaddress')),
-			&getParam(1, undef, $q->param('cidr')),
-			&getParam(1, '', $q->param('descr')),
-			&getParam(1, 0, $q->param('state')),
-			&getParam(1, 0, $q->param('tmplID')),
-			&getParam(1, 0, $q->param('defSubnetSize')),
-			&getParam(1, 0, $q->param('forceState')),
+			&getParam(1, 0, 'netID'),
+			&getParam(1, undef, 'rootID'),
+			&getParam(1, undef, 'netaddress'),
+			&getParam(1, undef, 'cidr'),
+			&getParam(1, '', 'descr'),
+			&getParam(1, 0, 'state'),
+			&getParam(1, 0, 'tmplID'),
+			&getParam(1, 0, 'defSubnetSize'),
+			&getParam(1, 0, 'forceState'),
+			0,
+			0,
+			&getParam(1, '', 'tags'),
 		)) {
 			$q->delete('func'); $q->param('func', 'showAllNets');
 		} else {
 			$q->delete('func'); $q->param('func', 'addNet');
 		}
 	}
-	elsif ($q->param('abortAddRoot')) {
+	elsif (&getParam(1, 0, 'abortAddRoot')) {
 		$q->delete('func'); $q->param('func', 'showAllNets');
 	}
-	elsif ($q->param('abortAddNet')) {
-		if ($q->param('editNet')) {
+	elsif (&getParam(1, 0, 'abortAddNet')) {
+		if (&getParam(1, 0, 'editNet')) {
 			$q->delete('func'); $q->param('func', 'showNet');
 		} else {
 			$q->delete('func'); $q->param('func', 'showAllNets');
 		}
 	}
-	elsif ($q->param('checktAddNet')) {
+	elsif (&getParam(1, 0, 'checktAddNet')) {
 		&checkNet(
-			&getParam(1, undef, $q->param('netaddress')), 
-			&getParam(1, undef, $q->param('cidr')),
+			&getParam(1, undef, 'netaddress'), 
+			&getParam(1, undef, 'cidr'),
 		);
-		if ($q->param('editNet')) {
+		if (&getParam(1, 0, 'editNet')) {
 			$q->delete('func'); $q->param('func', 'editNet');
 		} else {
 			$q->delete('func'); $q->param('func', 'addNet');
 		}
 	}
-	elsif ($q->param('submitImportASNRoutes')) {
+	elsif (&getParam(1, 0, 'submitImportASNRoutes')) {
 		if (
 			&importASNRoutes(
-				&getParam(1, undef, $q->param('asn'))
+				&getParam(1, undef, 'asn')
 			)
 		) {
 			$q->delete('func'); $q->param('func', 'showAllNets');
@@ -182,213 +196,233 @@ sub main {
 			$q->delete('func'); $q->param('func', 'importASNRoutes');
 		}
 	}
-	elsif ($q->param('submitImpDNSTrans')) {
+	elsif (&getParam(1, 0, 'submitImpDNSTrans')) {
 		if (&importDNSTrans()) {
 			$q->delete('func'); $q->param('func', 'showAllNets');
 		} else {
 			$q->delete('func'); $q->param('func', 'importDNS');
 		}
 	}
-	elsif ($q->param('submitImpDNSLocal')) {
+	elsif (&getParam(1, 0, 'submitImpDNSLocal')) {
 		if (&importDNSLocal()) {
 			$q->delete('func'); $q->param('func', 'showAllNets');
 		} else {
 			$q->delete('func'); $q->param('func', 'importDNS');
 		}
 	}
-	elsif ($q->param('submitImpConfig')) {
+	elsif (&getParam(1, 0, 'submitImpConfig')) {
 		if (&importConfig()) {
 			$q->delete('func'); $q->param('func', 'showAllNets');
 		} else {
 			$q->delete('func'); $q->param('func', 'importConfig');
 		}
 	}
-	elsif ($q->param('impCSVChangeSep')) {
+	elsif (&getParam(1, 0, 'impCSVChangeSep')) {
 		$q->delete('func'); $q->param('func', 'importConfig');
 	}
-	elsif ($q->param('impCSVChangeType')) {
+	elsif (&getParam(1, 0, 'impCSVChangeType')) {
 		$q->delete('func'); $q->param('func', 'importConfig');
 	}
-	elsif ($q->param('abortImpCSV')) {
+	elsif (&getParam(1, 0, 'abortImpCSV')) {
 		$q->delete('func'); $q->param('func', 'showAllNets');
 	}
-	elsif ($q->param('submitImpCSV')) {
+	elsif (&getParam(1, 0, 'submitImpCSV')) {
 		if (&importCSV()) {
 			$q->delete('func'); $q->param('func', 'showAllNets');
 		} else {
 			$q->delete('func'); $q->param('func', 'importConfig');
 		}
 	}
-	elsif ($q->param('editPluginConf')) {
+	elsif (&getParam(1, 0, 'editPluginConf')) {
 		$q->delete('func'); $q->param('func', 'showPluginConf');
 	}
-	elsif ($q->param('editNet')) {
+	elsif (&getParam(1, 0, 'editNet')) {
 		$q->delete('func'); $q->param('func', 'editNet');
 	}
-	elsif ($q->param('splitNet')) {
+	elsif (&getParam(1, 0, 'splitNet')) {
 		$q->delete('func'); $q->param('func', 'splitNet');
 	}
-	elsif ($q->param('submitSplitNet')) {
+	elsif (&getParam(1, 0, 'submitSplitNet')) {
 		&splitNet(
-			&getParam(1, undef, $q->param('netID')),
-			&getParam(1, undef, $q->param('splitCidr')),
-			&getParam(1, '', $q->param('descrTemplate')),
-			&getParam(1, 0, $q->param('state')),
-			&getParam(1, 0, $q->param('tmplID')),
-			&getParam(1, 0, $q->param('delParentNet')),
+			&getParam(1, undef, 'netID'),
+			&getParam(1, undef, 'splitCidr'),
+			&getParam(1, '', 'descrTemplate'),
+			&getParam(1, 0, 'state'),
+			&getParam(1, 0, 'tmplID'),
+			&getParam(1, 0, 'delParentNet'),
 		);
 		$q->delete('func'); $q->param('func', 'showAllNets');
 	}
-	elsif ($q->param('submitCombineNets')) {
+	elsif (&getParam(1, 0, 'submitCombineNets')) {
 		&combineNets();
 		$q->delete('func'); $q->param('func', 'showAllNets');
 	}
-	elsif ($q->param('abortCombineNets')) {
+	elsif (&getParam(1, 0, 'abortCombineNets')) {
 		$q->delete('func'); $q->param('func', 'showAllNets');
 	}
-	elsif ($q->param('abortSplitNet')) {
+	elsif (&getParam(1, 0, 'abortSplitNet')) {
 		$q->delete('func'); $q->param('func', 'showNet');
 	}
-	elsif ($q->param('combineNets')) {
-		my @nets	= $q->param('selectedNetworks');
-		if ($#nets > 0) {
+	elsif (&getParam(1, 0, 'combineNets')) {
+		my $nets	= &getParam(0, [], 'selectedNetworks');
+		if ($#{$nets} > 0) {
 			$q->delete('func'); $q->param('func', 'combineNets');
 		}
 	}
-	elsif ($q->param('abortEditRoot')) {
+	elsif (&getParam(1, 0, 'abortEditRoot')) {
 		$q->delete('func'); $q->param('func', 'showRoot');
 	}
-	elsif ($q->param('editRoot')) {
+	elsif (&getParam(1, 0, 'editRoot')) {
 		$q->delete('func'); $q->param('func', 'editRoot');
 	}
-	elsif ($q->param('abortDelRoot')) {
+	elsif (&getParam(1, 0, 'abortDelRoot')) {
 		$q->delete('func'); $q->param('func', 'showRoot');
 	}
-	elsif ($q->param('abortDelNet')) {
+	elsif (&getParam(1, 0, 'abortDelNet')) {
 		$q->delete('func'); $q->param('func', 'showNet');
 	}
-	elsif ($q->param('delRoot')) {
-		if ($q->param('commitDelRoot')) {
+	elsif (&getParam(1, 0, 'delRoot')) {
+		if (&getParam(1, 0, 'commitDelRoot')) {
 			&delRoot(
-				&getParam(1, undef, $q->param('rootID'))
+				&getParam(1, undef, 'rootID')
 			);
 			$q->delete('func'); $q->param('func', 'showAllNets');
 		} else {
 			$q->delete('func'); $q->param('func', 'delRoot');
 		}
 	}
-	elsif ($q->param('delNet')) {
-		if ($q->param('commitDelNet')) {
-			&delNet(
-				&getParam(1, undef, $q->param('netID')), 
-				((defined $q->param('withSubnets')) ? 1 : 0)
-			);
-			$q->delete('func'); $q->param('func', 'showAllNets');
+	elsif (&getParam(1, 0, 'delNet')) {
+		if (&getParam(1, 0, 'commitDelNet')) {
+			my $networkLock	= &getParam(1, 0, 'networkLock');
+			if (defined $networkLock && int($networkLock) ne $networkLock) {
+				&warnl(sprintf(_gettext("Network lock time must be provided in seconds not '%s'"), $networkLock));
+				$q->delete('func'); $q->param('func', 'delNet');
+			} else {
+				&delNet(
+					&getParam(1, undef, 'netID'), 
+					((&getParam(1, 0, 'withSubnets')) ? 1 : 0),
+					0,
+					&getParam(1, 0, 'networkLock')
+				);
+				$q->delete('func'); $q->param('func', 'showAllNets');
+			}
 		} else {
 			$q->delete('func'); $q->param('func', 'delNet');
 		}
 	}
-	elsif ($q->param('reduceRoot')) {
+	elsif (&getParam(1, 0, 'reduceRoot')) {
 		&expand(
 			'-', 
 			'root', 
-			&getParam(1, undef, $q->param('reduceRoot'))
+			&getParam(1, undef, 'reduceRoot')
 		);
 		$session->param('currNet', '');
 		$session->param(
 			'currRootID', 
-			&getParam(1, undef, $q->param('reduceRoot'))
+			&getParam(1, undef, 'reduceRoot')
 		);
 	}
-	elsif ($q->param('expandRoot')) {
+	elsif (&getParam(1, 0, 'expandRoot')) {
 		&expand(
 			'+', 
 			'root', 
-			&getParam(1, undef, $q->param('expandRoot'))
+			&getParam(1, undef, 'expandRoot')
 		);
 		$session->param('currNet', '');
 		$session->param(
 			'currRootID', 
-			&getParam(1, undef, $q->param('expandRoot'))
+			&getParam(1, undef, 'expandRoot')
 		);
 	}
-	elsif ($q->param('reduceNetwork')) {
+	elsif (&getParam(1, 0, 'reduceNetwork')) {
 		&expand(
 			'-', 
 			'network', 
-			&getParam(1, undef, $q->param('reduceNetwork')), 
-			&getParam(1, undef, $q->param('rootID'))
+			&getParam(1, undef, 'reduceNetwork'), 
+			&getParam(1, undef, 'rootID')
 		);
 		$session->param(
 			'currNet', 
-			&getParam(1, undef, $q->param('reduceNetwork'))
+			&getParam(1, undef, 'reduceNetwork')
 		);
 		$session->param(
 			'currRootID', 
-			&getParam(1, undef, $q->param('rootID'))
+			&getParam(1, undef, 'rootID')
 		);
 	}
-	elsif ($q->param('expandNetwork')) {
+	elsif (&getParam(1, 0, 'expandNetwork')) {
 		&expand(
 			'+', 
 			'network', 
-			&getParam(1, undef, $q->param('expandNetwork')), 
-			&getParam(1, undef, $q->param('rootID'))
+			&getParam(1, undef, 'expandNetwork'), 
+			&getParam(1, undef, 'rootID')
 		);
 		$session->param(
 			'currNet', 
-			&getParam(1, undef, $q->param('expandNetwork'))
+			&getParam(1, undef, 'expandNetwork')
 		);
 		$session->param(
 			'currRootID', 
-			&getParam(1, undef, $q->param('rootID'))
+			&getParam(1, undef, 'rootID')
 		);
 	}
-	elsif ($q->param('closeTree')) {
+	elsif (&getParam(1, 0, 'closeTree')) {
 		&expand('-', 'ALL', 'ALL');
 	}
-	elsif ($q->param('jumpToButton')) {
+	elsif (&getParam(1, 0, 'jumpToButton')) {
 		&expandTo(
-			&getParam(1, undef, $q->param('rootIDJump')), 
-			&getParam(1, undef, $q->param('jumpTo'))
+			&getParam(1, undef, 'rootIDJump'), 
+			&getParam(1, undef, 'jumpTo')
 		);
 	}
-	elsif ($q->param('genRandBranch')) {
+	elsif (&getParam(1, 0, 'genRandBranch')) {
 		&genRandBranch();
 		$q->delete('func'); $q->param('func', 'showAllNets');
 	}
-	elsif ($q->param('finishEditTree')) {
+	elsif (&getParam(1, 0, 'finishEditTree')) {
 		$q->delete('editTree');
 	}
-	elsif ($q->param('copyNetsTo')) {
+	elsif (&getParam(1, 0, 'copyNetsTo')) {
 		&copyNetsTo(
-			&getParam(1, undef, $q->param('copyToRootID')), 
-			[$q->param('selectedNetworks')], 
+			&getParam(1, undef, 'copyToRootID'), 
+			&getParam(0, [], 'selectedNetworks'), 
 			0
 		);
 	}
-	elsif ($q->param('moveNetsTo')) {
+	elsif (&getParam(1, 0, 'moveNetsTo')) {
 		&copyNetsTo(
-			&getParam(1, undef, $q->param('copyToRootID')), 
-			[$q->param('selectedNetworks')], 
+			&getParam(1, undef, 'copyToRootID'), 
+			&getParam(0, [], 'selectedNetworks'), 
 			1
 		);
 	}
-	elsif ($q->param('deleteNets')) {
-		&delNets([$q->param('selectedNetworks')]);
+	elsif (&getParam(1, 0, 'deleteNets')) {
+		&delNets(&getParam(0, [], 'selectedNetworks'));
 	}
-	elsif ($q->param('searchButton')) {
-		&search();
-		$q->delete('func'); $q->param('func', 'search');
+	elsif (&getParam(1, 0, 'searchButton')) {
+		if ((&getParam(1, '', 'func') eq 'getFreeSubnetsFromSearch')) {
+			&searchAndGetFreeSubnets();
+			$t->{V}->{searchResult}	= $t->{V}->{freeSubnets};
+		} else {
+			&search();
+		}
 	}
-	elsif ($q->param('compareButton')) {
+	elsif (&getParam(1, 0, 'compareButton')) {
 		&compare();
 		$q->delete('func'); $q->param('func', 'showAllNets');
 	}
-	elsif ($q->param('newTmpl')) {
-		if ($q->param('tmplName')) {
+	elsif (&getParam(1, 0, 'newTmpl')) {
+		my $tmplName	= &getParam(1, '', 'tmplName');
+
+		my $exists		= 0;
+		if ($tmplName) {
+			$exists		= &checkIfExistsInDB('template', 'name', $tmplName);
+			&warnl("Template '$tmplName' already exists!") if $exists;
+		}	
+
+		if ($tmplName && !$exists) {
 			$q->delete('tmplID');
-			my $tmplName	= &getParam(1, '', $q->param('tmplName'));
+			my $tmplName	= &getParam(1, '', 'tmplName');
 			$tmplName			=~ s/[<>&"']//g;
 			$q->delete('tmplName'), $q->param('tmplName', $tmplName);
 			$q->delete('func'); $q->param('func', 'editTmpl');
@@ -397,65 +431,76 @@ sub main {
 			$q->delete('func'); $q->param('func', 'showTemplates');
 		}
 	}
-	elsif ($q->param('editNetTypeTmpl')) {
-		unless (defined $q->param('tmplID')) {
+	elsif (&getParam(1, 0, 'editNetTypeTmpl')) {
+		unless (defined &getParam(1, undef, 'tmplID')) {
 			$q->delete('func'); $q->param('func', 'showTemplates');
 		} else {
 			$q->delete('func'); $q->param('func', 'editTmpl');
 		}
 	}
-	elsif ($q->param('abortEditTmpl')) {
+	elsif (&getParam(1, 0, 'editTemplName')) {
+		&changeTmplName(&getParam(1, undef, 'tmplID', 1), &getParam(1, 0, 'tmplName'));
+		$q->delete('func'); $q->param('func', 'editTmpl');
+	}
+	elsif (&getParam(1, 0, 'abortEditTmpl')) {
 		$q->delete('func'); $q->param('func', 'showTemplates');
 	}
-	elsif ($q->param('submitAddTmplEntry')) {
+	elsif (&getParam(1, 0, 'submitAddTmplEntry')) {
 		my $tmplID	= &saveTmpl();
 		$q->delete('tmplID'); $q->param('tmplID', $tmplID);
 		$q->delete('func'); $q->param('func', 'editTmpl');
 	}
-	elsif ($q->param('submitEditTmplEntry')) {
+	elsif (&getParam(1, 0, 'submitEditTmplEntry')) {
 		my $tmplID	= &saveTmpl(1);
 		$q->delete('tmplID'); $q->param('tmplID', $tmplID);
 		$q->delete('func'); $q->param('func', 'editTmpl');
 	}
-	elsif ($q->param('submitDeleteTmplEntry')) {
+	elsif (&getParam(1, 0, 'submitDeleteTmplEntry')) {
 		my $tmplID	= &saveTmpl(2);
 		$q->delete('tmplID'); $q->param('tmplID', $tmplID);
 		$q->delete('func'); $q->param('func', 'editTmpl');
 	}
-	elsif ($q->param('delTmpl')) {
-		if ($q->param('commitDelTmpl')) {
-			&delTmpl(&getParam(1, undef, $q->param('tmplID')));
+	elsif (&getParam(1, 0, 'delTmpl')) {
+		if (&getParam(1, 0, 'commitDelTmpl')) {
+			&delTmpl(&getParam(1, undef, 'tmplID'));
 			$q->delete('func'); $q->param('func', 'showTemplates');
 		}
-		elsif ($q->param('abortDelTmpl')) {
+		elsif (&getParam(1, 0, 'abortDelTmpl')) {
 			$q->delete('func'); $q->param('func', 'showTemplates');
 		} else {
-			if ($q->param('tmplID')) {
+			if (&getParam(1, 0, 'tmplID')) {
 				$q->delete('func'); $q->param('func', 'delTmpl');
 			} else {
 				$q->delete('func'); $q->param('func', 'showTemplates');
 			}
 		}
 	}
-	elsif ($q->param('newGroup')) {
-		if ($q->param('groupName')) {
+	elsif (&getParam(1, 0, 'newGroup')) {
+		my $groupName	= &getParam(1, '', 'groupName');
+		#$groupName		=~ s/[<>&"']//g;
+
+		my $exists		= 0;
+		if ($groupName) {
+			$exists		= &checkIfExistsInDB('group', 'name', $groupName);
+			&warnl("Group '$groupName' already exists!") if $exists;
+		}	
+
+		if ($groupName && !$exists) {
 			$q->delete('groupID');
-			my $groupName	= &getParam(1, '', $q->param('groupName'));
-			$groupName		=~ s/[<>&"']//g;
 			$q->delete('groupName'), $q->param('groupName', $groupName);
 			$q->delete('func'); $q->param('func', 'editGroup');
 		} else {
 			$q->delete('func'); $q->param('func', 'showGroups');
 		}
 	}
-	elsif ($q->param('editGroup')) {
-		unless (defined $q->param('groupID')) {
+	elsif (&getParam(1, 0, 'editGroup')) {
+		unless (defined &getParam(1, undef, 'groupID')) {
 			$q->delete('func'); $q->param('func', 'showGroups');
 		} else {
 			$q->delete('func'); $q->param('func', 'editGroup');
 		}
 	}
-	elsif ($q->param('submitEditGroup')) {
+	elsif (&getParam(1, 0, 'submitEditGroup')) {
 		my $groupID	= &saveGroup();
 		if (defined $groupID) {
 			$q->delete('groupID'); $q->param('groupID', $groupID);
@@ -464,20 +509,21 @@ sub main {
 			$q->delete('func'); $q->param('func', 'editGroup');
 		}
 	}
-	elsif ($q->param('abortEditGroup')) {
+	elsif (&getParam(1, 0, 'abortEditGroup')) {
 		$q->delete('func'); $q->param('func', 'showGroups');
 	}
-	elsif ($q->param('delGroup')) {
-		if ($q->param('commitDelGroup')) {
-			&delGroup(&getParam(1, undef, $q->param('groupID')));
+	elsif (&getParam(1, 0, 'delGroup')) {
+		if (&getParam(1, 0, 'commitDelGroup')) {
+			&delGroup(&getParam(1, undef, 'groupID'));
 			$q->delete('func'); $q->param('func', 'showGroups');
 		}
-		elsif ($q->param('abortDelGroup')) {
+		elsif (&getParam(1, 0, 'abortDelGroup')) {
 			$q->delete('func'); $q->param('func', 'showGroups');
 		} else {
-			if ($q->param('groupID')) {
-				my $groupName	= &groupID2Name(&getParam(1, undef, $q->param('groupID')));
+			if (&getParam(1, 0, 'groupID')) {
+				my $groupName	= &groupID2Name(&getParam(1, undef, 'groupID'));
 				if ($groupName eq 'Administrator') {
+					&warnl(_gettext("You cannot delete the administrator group!"));
 					$q->delete('func'); $q->param('func', 'showGroups');
 				} else {
 					$q->delete('func'); $q->param('func', 'delGroup');
@@ -487,25 +533,32 @@ sub main {
 			}
 		}
 	}
-	elsif ($q->param('newUser')) {
-		if ($q->param('userName')) {
+	elsif (&getParam(1, 0, 'newUser')) {
+		my $userName	= &getParam(1, '', 'userName');
+		#$userName			=~ s/[<>&"']//g;
+
+		my $exists		= 0;
+		if ($userName) {
+			$exists		= &checkIfExistsInDB('user', 'username', $userName);
+			&warnl("User '$userName' already exists!") if $exists;
+		}	
+
+		if ($userName && !$exists) {
 			$q->delete('userID');
-			my $userName	= &getParam(1, '', $q->param('userName'));
-			$userName			=~ s/[<>&"']//g;
 			$q->delete('userName'), $q->param('userName', $userName);
 			$q->delete('func'); $q->param('func', 'editUser');
 		} else {
 			$q->delete('func'); $q->param('func', 'showUsers');
 		}
 	}
-	elsif ($q->param('editUser')) {
-		unless (defined $q->param('userID')) {
+	elsif (&getParam(1, 0, 'editUser')) {
+		unless (defined &getParam(1, undef, 'userID')) {
 			$q->delete('func'); $q->param('func', 'showUsers');
 		} else {
 			$q->delete('func'); $q->param('func', 'editUser');
 		}
 	}
-	elsif ($q->param('submitEditUser')) {
+	elsif (&getParam(1, 0, 'submitEditUser')) {
 		my $userID	= &saveUser();
 		if (defined $userID) {
 			$q->delete('userID'); $q->param('userID', $userID);
@@ -514,42 +567,42 @@ sub main {
 			$q->delete('func'); $q->param('func', 'editUser');
 		}
 	}
-	elsif ($q->param('abortEditUser')) {
+	elsif (&getParam(1, 0, 'abortEditUser')) {
 		$q->delete('func'); $q->param('func', 'showUsers');
 	}
-	elsif ($q->param('delUser')) {
-		if ($q->param('commitDelUser')) {
-			&delUser(&getParam(1, undef, $q->param('userID')));
+	elsif (&getParam(1, 0, 'delUser')) {
+		if (&getParam(1, 0, 'commitDelUser')) {
+			&delUser(&getParam(1, undef, 'userID'));
 			$q->delete('func'); $q->param('func', 'showUsers');
 		}
-		elsif ($q->param('abortDelUser')) {
+		elsif (&getParam(1, 0, 'abortDelUser')) {
 			$q->delete('func'); $q->param('func', 'showUsers');
 		} else {
-			if ($q->param('userID')) {
+			if (&getParam(1, 0, 'userID')) {
 				$q->delete('func'); $q->param('func', 'delUser');
 			} else {
 				$q->delete('func'); $q->param('func', 'showUsers');
 			}
 		}
 	}
-	elsif ($q->param('showSubnets')) {
+	elsif (&getParam(1, 0, 'showSubnets')) {
 		$q->delete('func'); $q->param('func', 'showSubnets');
 	}
-	elsif ($q->param('abortShowSubnets')) {
+	elsif (&getParam(1, 0, 'abortShowSubnets')) {
 		$q->delete('func'); $q->param('func', 'showNet');
 	}
-	elsif ($q->param('checkDB')) {
+	elsif (&getParam(1, 0, 'checkDB')) {
 		&checkDB();
 	}
-	elsif ($q->param('submitshowPlugins')) {
+	elsif (&getParam(1, 0, 'submitshowPlugins')) {
 		&updatePluginDB();
 		$q->delete('func'); $q->param('func', 'showPlugins');
 	}
-	elsif ($q->param('editPluginGlobConf')) {
+	elsif (&getParam(1, 0, 'editPluginGlobConf')) {
 		$q->delete('func'); $q->param('func', 'showPluginGlobConf');
 	}
-	elsif ($q->param('submitPluginConfig')) {
-		my $global	= &getParam(1, 0, $q->param('global'));
+	elsif (&getParam(1, 0, 'submitPluginConfig')) {
+		my $global	= &getParam(1, 0, 'global');
 		&mkPluginConfig($global);
 		if ($global) {
 			$q->delete('func'); $q->param('func', 'showPlugins');
@@ -557,8 +610,8 @@ sub main {
 			$q->delete('func'); $q->param('editNet', 'Bearbeiten'); $q->param('func', 'editNet');
 		}
 	}
-	elsif ($q->param('abortPluginConfig')) {
-		my $global	= &getParam(1, 0, $q->param('global'));
+	elsif (&getParam(1, 0, 'abortPluginConfig')) {
+		my $global	= &getParam(1, 0, 'global');
 		if ($global) {
 			$q->delete('func'); $q->param('func', 'showPlugins');
 		} else {
@@ -566,35 +619,56 @@ sub main {
 			$q->delete('editNet'); $q->param('editNet', 1);
 		}
 	}
-	elsif ($q->param('abortShowSettings')) {
+	elsif (&getParam(1, 0, 'abortShowSettings')) {
 		$q->delete('func'); $q->param('func', 'showAllNets');
 	}
-	elsif ($q->param('commitChOwnPW')) {
+	elsif (&getParam(1, 0, 'commitChOwnPW')) {
 		unless (&chOwnPW()) {
 			$q->param('changeOwnPW', 1);
 		}
 	}
-	elsif ($q->param('commitViewSettings')) {
+	elsif (&getParam(1, 0, 'commitShowAuditLogs')) {
+			$q->delete('func'); $q->param('func', 'showAuditLogs');
+	}
+	elsif (&getParam(1, 0, 'showAuditLogsLeft')) {
+			$q->delete('func'); $q->param('func', 'showAuditLogs');
+	}
+	elsif (&getParam(1, 0, 'showAuditLogsRight')) {
+			$q->delete('func'); $q->param('func', 'showAuditLogs');
+	}
+	elsif (&getParam(1, 0, 'commitViewSettings')) {
 		unless (&updSettings()) {
 			$q->param('showViewSettings', 1);
 		}
 		&HaCi::GUI::init::setUserVars();
 	}
-	elsif ($q->param('searchAndGetFreeSubnets')) {
+	elsif (&getParam(1, 0, 'searchAndGetFreeSubnets')) {
 		&searchAndGetFreeSubnets();
 	}
+	elsif (&getParam(1, 0, 'getFreeSubnets')) {
+		my $netID				= &getParam(1, undef, 'netID');
+		my $subnetSize	= &getParam(1, undef, 'subnetSize');
+		my $amount			= &getParam(1, undef, 'amount');
+		&getFreeSubnets($netID, 0, $subnetSize, $amount);
+	}
+	elsif (&getParam(1, 0, 'exportSubnets')) {
+		my $rootID	= &getParam(1, undef, 'rootID');
+		my $netID		= &getParam(1, undef, 'netID');
+		my $ID			= (defined $rootID) ? $rootID : $netID;
+		&exportSubnets($ID, (defined $rootID) ? 1 : 0);
+	}
 	
-	if (defined $q->param('func')) {
-		if ($q->param('func') eq 'addNet') {
-			my $roots	= &getRoots();
+	if (defined &getParam(1, 0, 'func')) {
+		if (&getParam(1, '', 'func') eq 'addNet') {
+			my $roots	= &getRoots(0);
 			if ($#{$roots} == -1) {
 				&warnl("Cannot find any Root! Please create some before you create networks\n");
 				$q->delete('func'); $q->param('func', 'addRoot');
 			}
 		}
-		elsif ($q->param('func') eq 'addNet' && $q->param('networkDec') && $q->param('rootID') && !($q->param('editNet'))) {
-			my $rootID			= &getParam(1, undef, $q->param('rootID'));
-			my $networkDec	= &getParam(1, undef, $q->param('networkDec'));
+		if (&getParam(1, '', 'func') eq 'addNet' && &getParam(1, 0, 'networkDec') && &getParam(1, 0, 'rootID') && !(&getParam(1, 0, 'editNet'))) {
+			my $rootID			= &getParam(1, undef, 'rootID');
+			my $networkDec	= &getParam(1, undef, 'networkDec');
 			my $ipv6				= &rootID2ipv6($rootID);
 			$networkDec			= Math::BigInt->new($networkDec) if $ipv6;
 			my $parent			= &getNetworkParentFromDB($rootID, $networkDec, $ipv6);
@@ -611,8 +685,12 @@ sub main {
 					$q->delete('func'); $q->param('func', 'showAllNets');
 				}
 			}
+
+			unless (&checkNetworkLock($rootID, $networkDec, $ipv6)) {
+				$q->delete('func'); $q->param('func', 'showAllNets');
+			}
 		}
-		elsif ($q->param('func') eq 'flushCache') {
+		elsif (&getParam(1, '', 'func') eq 'flushCache') {
 			&flushACLCache();
 			&flushNetCache();
 			$q->delete('func'); $q->param('func', 'showAllNets');
@@ -620,9 +698,9 @@ sub main {
 	}
 	
 	$HaCi::GUI::init::t->{V}->{jumpTo}  = $session->param('currRootID') . '-' . $session->param('currNet') if $session->param('currRootID') && $session->param('currNet');
-	&HaCi::GUI::main::start();
+	&HaCi::GUI::main::start() unless $conf->{var}->{soap} || &getParam(1, 0, 'exportSubnet');
 
-	if ($q->param('newWindow') && $q->param('newWindow') eq 'showStatus') {
+	if (&getParam(1, 0, 'newWindow') && &getParam(1, '', 'newWindow') eq 'showStatus') {
 		$t->{V}->{page}  = 'showStatus';
 	} else {
 		$t->{V}->{page}  = 'main';
@@ -631,36 +709,44 @@ sub main {
 
 sub expandTo {
 	warn 'SUB: ' . (caller(0))[3] . ' (' . (caller(1))[3] . ")\n" if $conf->{var}->{showsubs};
-	my $rootID				= shift;
-	my $ipaddress			= shift || '';
-	my $ipv6					= &rootID2ipv6($rootID);
-	$ipaddress				=~ s/[^\w\.\:]//g;
+	my $rootID	= shift;
+	my $network	= shift || '';
+	my $ipv6		= &rootID2ipv6($rootID);
+	$network	=~ s/[^\w\.\:\/]//g;
 
-	return 0 unless $ipaddress;
+	return 0 unless $network;
 
-	unless (&checkSpelling_IP($ipaddress, $ipv6)) {
-		warn "Cannot jump to. No IP Address: '$ipaddress'\n";
+	unless (&checkSpelling_Net($network, $ipv6) || &checkSpelling_IP($network, $ipv6)) {
+		warn "Cannot jump to. No IP Address / network: '$network'\n";
 		return 0;
 	}
 
-	my $networkDec		= ($ipv6) ? &netv62Dec($ipaddress . '/128') : &net2dec($ipaddress . '/32');
+	$network					.= ($ipv6) ? '/128' : '/32' unless $network =~ /\/\d+$/;
+	my $networkDec		= ($ipv6) ? &netv62Dec($network) : &net2dec($network);
 	my $s							= $HaCi::HaCi::session;
 	my $expands				= $s->param('expands') || {};
-	
 	my $networkTable	= $conf->{var}->{TABLES}->{network};
 	unless (defined $networkTable) {
 		warn "Cannot jump to network. DB Error\n";
 		return 0;
 	}
 
-	my $bestMatch	= 0;
+	my $ipv6ID		= ($ipv6) ? &netv6Dec2ipv6ID($networkDec) : '';
+	my $netID			= &getNetID($rootID, $networkDec, $ipv6ID);
+	my $bestMatch	= (defined $netID) ? $networkDec : 0;
+	unless ($bestMatch) {
+		$networkDec		= &getDBNetworkBefore($rootID, $networkDec, $ipv6);
+		$netID				= &getNetID($rootID, $networkDec, $ipv6ID);
+		$bestMatch		= (defined $netID) ? $networkDec : 0;
+	}
+
 	while (my $parent	= &getNetworkParentFromDB($rootID, $networkDec, $ipv6)) {
 		my $parentDec	= $parent->{network};
 		&expand('+', 'network', $parentDec, $rootID) unless $expands->{network}->{$rootID}->{$parentDec};
 		$bestMatch	= $parentDec unless $bestMatch;
 		$networkDec	= $parentDec;
 	}
-	
+
 	if ($bestMatch) {
 		&expand('+', 'root', $rootID);
 		$session->param('jumpTo', 1);
@@ -693,6 +779,38 @@ sub hook {
 	};
 }
 
+sub getSession {
+	my $sessID	= shift;
+
+	eval {
+		$session	= CGI::Session->load($sessID);
+	};
+	if ($@ || !defined $session) {
+		warn $@ . ' ' . CGI::Session->errstr;
+		$conf->{var}->{authenticationError} = _gettext("Session is broken");
+		$conf->{var}->{relogin}	= 1;
+		#Do not return 0 if sessID is undef, but force to gen new session, so that an API call didn't fail
+		return 0 if defined $sessID && $sessID;
+	}
+
+	if ( defined $session && $session->is_expired ) {
+		$conf->{var}->{authenticationError} = _gettext("Session is expired");
+		$conf->{var}->{relogin}	= 1;
+		return 0;
+	}
+	if ( !defined $session || $session->is_empty ) {
+		if (defined $session) {
+			&debug("Session " . $session->id() . " is empty. Generating new...");
+		} else {
+			&debug("Session cannot be loaded! Generating new...");
+		}
+		&genNewSession();
+	}
+	$session->expire($conf->{static}->{misc}->{sessiontimeout});
+
+	return $session;
+}
+
 sub init {
 	warn 'SUB: ' . (caller(0))[3] . ' (' . (caller(1))[3] . ")\n" if $conf->{var}->{showsubs};
 	$q		= CGI->new(\&hook, $ENV{CONTENT_LENGTH});
@@ -707,27 +825,9 @@ sub init {
 	$pjx->DEBUG(0);
 
 	my %cookies = fetch CGI::Cookie;
-	my $sid			= &getParam(1, undef, $q->param('sid'));
 	my $sessID	= (exists $cookies{CGISESSID}) ? $cookies{CGISESSID}->value : undef;
 
-	eval {
-		$session	= CGI::Session->load($sessID) or die CGI::Session->errstr;
-	};
-	if ($@) {
-		warn $@;
-		$conf->{var}->{authenticationError} = _gettext("Session is broken");
-		$conf->{var}->{relogin}	= 1;
-		return;
-	}
-	if ( $session->is_expired ) {
-		$conf->{var}->{authenticationError} = _gettext("Session is expired");
-		$conf->{var}->{relogin}	= 1;
-	}
-	my @params	= $q->param;
-	if ( $session->is_empty && $#params != -1) {
-		$session = $session->new($sid);
-	}
-	$session->expire($conf->{static}->{misc}->{sessiontimeout});
+	return unless &getSession($sessID);
 
 	($aclCacheHandle, $netCacheHandle)	= &initCache();
 	if (defined $netCacheHandle) {
@@ -743,10 +843,10 @@ sub init {
 
 	&initTables();
 
-	if ($q->param('locale')) {
-		$session->param('locale', $q->param('locale'));
+	if (&getParam(1, 0, 'locale')) {
+		$session->param('locale', &getParam(1, '', 'locale'));
 	}
-	$q->delete('jumpTo') if defined $q->param('jumpTo') && $q->param('jumpTo') eq '<IP Adresse>';
+	$q->delete('jumpTo') if &getParam(1, 0, 'jumpTo') && &getParam(1, '', 'jumpTo') eq '<IP Adresse>';
 }
 
 sub soapInit {
@@ -761,63 +861,70 @@ sub soapInit {
 
 sub genNewSession {
 	warn 'SUB: ' . (caller(0))[3] . ' (' . (caller(1))[3] . ")\n" if $conf->{var}->{showsubs};
-	my $sid		= &getParam(1, undef, $q->param('sid'));
-	$session	= CGI::Session->new($sid) or die CGI::Session->errstr;
+	$session	= CGI::Session->new() or die CGI::Session->errstr;
 	$session->expire($conf->{static}->{misc}->{sessiontimeout});
 	
-	if ($q->param('locale')) {
-		$session->param('locale', $q->param('locale'));
+	if (ref($q) && &getParam(1, 0, 'locale')) {
+		$session->param('locale', &getParam(1, '', 'locale'));
 	}
 }
 
 sub authentication {
 	warn 'SUB: ' . (caller(0))[3] . ' (' . (caller(1))[3] . ")\n" if $conf->{var}->{showsubs};
 
-	use HaCi::Authentication::internal;
-	my $authAdmin	= new HaCi::Authentication::internal;
-	$authAdmin->session($session);
-	$authAdmin->init();
-	
-	my $auth;
-	if (($q->param('login') && $q->param('username') eq 'admin') || ($session->param('username') && $session->param('username') eq 'admin') ) {
-		$auth	= $authAdmin;
-	} else {
-		my $temp						= (exists $conf->{user}->{auth}->{authmodule} && $conf->{user}->{auth}->{authmodule}) ? $conf->{user}->{auth}->{authmodule} : 'internal';
-		$temp								= 'internal' if $temp eq 'HaCi';	# fix to be backward compatible 
-		my $authModule			= "HaCi::Authentication::$temp";
-		my $authModuleFile	= $conf->{static}->{path}->{authmodules} . '/' . $temp . '.pm';
-		eval {
-			require $authModuleFile;
-		};
-		if ($@) {
-			$conf->{var}->{authenticationError}	= _gettext("Authentication Error");
-			&warnl($@);
-			return 0;
-		}
-		$auth	= $authModule->new();
-	}
-	$auth->session($session);
-	
-	if ($q->param('login')) {
-		my $user	= &getParam(1, undef, $q->param('username'));
-		my $pass	= &getParam(1, undef, $q->param('password'));
+	my $user				= shift;
+	my $pass				= shift;
+	my $bLogin			= shift;
+	my $sessionUser	= shift;
+
+	if ($bLogin) {
 		unless (defined $user) {
 			$conf->{var}->{authenticationError}	= _gettext("No User given");
 			return 0;
 		}
-		
-		$auth->user($user);
-		$auth->pass($pass);
-		
-		if ($auth->authenticate()) {
-			$conf->{var}->{authenticated}	= 1;
-			&updateSettingsInSession();
-			return 1;
-		} else {
-			return 0;
+
+		my @authModules	= (($bLogin && $user eq 'admin') || ($sessionUser eq 'admin'))
+			? 'internal'
+			: (exists $conf->{user}->{auth}->{authmodule} && $conf->{user}->{auth}->{authmodule} ne '') ? split(/\s*,\s*/, $conf->{user}->{auth}->{authmodule}) : ('internal');
+
+		my $authErrors	= [];
+		foreach (@authModules) {
+			my $authModule	= $_;
+			$authModule			= 'internal' if $authModule eq 'HaCi'; # fix to be backward compatible
+
+			my $authModuleFile	= $conf->{static}->{path}->{authmodules} . '/' . $authModule . '.pm';
+			eval {
+				require $authModuleFile;
+			};
+			if ($@) {
+				$conf->{var}->{authenticationError}	= _gettext("Authentication Error");
+				&warnl($@);
+				return 0;
+			}
+
+			my $auth	= "HaCi::Authentication::$authModule"->new();
+			$auth->session($session);
+			$auth->init() if $authModule eq 'internal';
+
+			$auth->user($user);
+			$auth->pass($pass);
+			
+			if ($auth->authenticate()) {
+				$conf->{var}->{authenticated}	= 1;
+				&updateSettingsInSession();
+				&debug("Successfully authenticated by '$authModule'") if 1;
+				return 1;
+			} else {
+				push @{$authErrors}, $conf->{var}->{authenticationError};
+				$conf->{var}->{authenticationError}	= '';
+				&debug("Not authenticated by '$authModule'") if 1;
+			}
 		}
+
+		$conf->{var}->{authenticationError}	= join('<br>', @{$authErrors});
+		return 0;
 	} else {
-		if ($auth->isAutenticated()) {
+		if (defined $session->param('authenticated') && $session->param('authenticated')) {
 			&debug("Allready authenticated") if 0;
 			$conf->{var}->{authenticated}	= 1;
 			return 1;
@@ -831,10 +938,19 @@ sub authentication {
 sub finalize {
 	warn 'SUB: ' . (caller(0))[3] . ' (' . (caller(1))[3] . ")\n" if $conf->{var}->{showsubs};
 	my $template	= $HaCi::GUI::init::t;
-	my $cookie		= $q->cookie(
-		-name		=> $session->name,
-		-value	=> $session->id 
-	);
+	my $cookie;
+	if (defined $session) {
+		$cookie	= $q->cookie(
+			-name		=> $session->name,
+			-value	=> $session->id 
+		);
+	} else {
+		&debug("No session available while finalizing. Setting session-id to 0");
+		$cookie	= $q->cookie(
+			-name		=> 'CGISESSID',
+			-value	=> 0
+		);
+	}
 
 	unless ($conf->{var}->{soap}) {
 		my $html_output	= '';
@@ -859,11 +975,12 @@ sub finalize {
 		}
 
 		print $q->Dump() if 0;
-		if (0) { map { warn " $_ -> " . $q->param($_); } $q->param; } # Dump
+		if (0) { map { warn " $_ -> " . &getParam(1, '', $_); } $q->param; } # Dump
 
 		print '<script>var aktiv = setTimeout("refresh()", 500);</script>' if $conf->{var}->{reloadPage};
 		print '<script>window.close();</script>' if $conf->{var}->{closePage};
 	}
+	undef $HaCi::HaCi::q;
 
 	# Cache Statistics
 	if (0) {
@@ -882,10 +999,12 @@ sub finalize {
 		warn "Cannot set Cache (aclCache!)\n" unless $aclCacheHandle->set('HASH', $aclCache);
 	}
 
-	$session->clear('jumpTo');
-	$session->clear('jumpToNet');
-	$session->clear('jumpToRoot');
-	$session->flush();
+	if (defined $session) {
+		$session->clear('jumpTo');
+		$session->clear('jumpToNet');
+		$session->clear('jumpToRoot');
+		$session->flush();
+	}
 
 	&warnl($conf->{var}->{authenticationError}) if $conf->{var}->{authenticationError} && $conf->{var}->{soap};
 	return &prWarnl(1) if $conf->{var}->{soap};
@@ -972,6 +1091,12 @@ sub checkRights {
 				/^search$/ ||
 				/^compare$/
 			) && !&checkRight('search') ||
+			(
+				/^getFreeSubnetsFromSearch$/
+			) && (!&checkRight('search') || !&checkRight('addNet')) ||
+			(
+				/^getFreeSubnets$/
+			) && (!&checkRight('search')) ||
 			(
 				/^submitImportASNRoutes$/ 
 			) && !&checkRight('impASNRoutes') ||
@@ -986,7 +1111,13 @@ sub checkRights {
 				/^submitImpConfig$/ ||
 				/^impCSVChangeSep$/ ||
 				/^submitImpCSV$/
-			) && !&checkRight('impConfig')
+			) && !&checkRight('impConfig') ||
+			(
+				/^showAditLogs$/ ||
+				/^showAditLogsLeft$/ ||
+				/^showAditLogsRight$/ ||
+				/^commitShowAuditLogs$/
+			) && !&checkRight('showAuditLogs')
 	}
 }
 
@@ -999,6 +1130,23 @@ sub delParam {
 	$q->delete($param);
 }
 
+sub checkIfExistsInDB {
+	warn 'SUB: ' . (caller(0))[3] . ' (' . (caller(1))[3] . ")\n" if $conf->{var}->{showsubs};
+
+	my $table	= shift;
+	my $col		= shift;
+	my $val		= shift;
+
+	my $table  = $conf->{var}->{TABLES}->{$table};
+	unless (defined $table) {
+		&warn("Cannot check. DB Error ($table)");
+		return 1;
+	}
+	my $DB  = ($table->search(['ID'], {$col => $val}))[0];
+
+	return (defined $DB) ? 1 : 0;
+}
+
 1;
 
 # vim:ts=2:sw=2:sws=2
