@@ -6,6 +6,7 @@ package HaCi::SOAP::Type::network;
 	_ATTR network			$string Network
 	_ATTR description	$string Description
 	_ATTR state				$string State
+	_ATTR tags				$string Tags
 
 =cut
 
@@ -14,13 +15,15 @@ sub new {
 	my $rootName		= shift || '';
 	my $network			= shift || '';
 	my $description	= shift || '';
-	my $state				= shift || 0;
+	my $state				= SOAP::Data->type(string => (shift || 0));
+	my $tags				= shift || '';
 
 	bless {
 		rootName		=> $rootName,
 		network			=> $network,
 		description	=> $description,
 		state				=> $state,
+		tags				=> $tags,
 	}, $class;
 }
 
